//
//  MyView.m
//  PhotoExample
//
//  Created by Bill Dudney on 10/19/08.
//  Copyright 2008 Gala Factory. All rights reserved.
//

#import "MyView.h"

@implementation MyView


- (void)awakeFromNib 
{
  transformed = [CALayer layer];
  transformed.frame = self.bounds;
  [self.layer addSublayer:transformed];

  CALayer *imageLayer = [CALayer layer];
  imageLayer.frame = CGRectMake(10.0f, 4.0f, 300.0f, 226.0f);
  imageLayer.transform = CATransform3DMakeRotation(20.0f * M_PI / 180.0f,
                                                   1.0f, 0.0f, 0.0f);
  imageLayer.contents = (id)[[UIImage imageNamed:@"IMG_0051.png"] CGImage];
  [transformed addSublayer:imageLayer];
  
  imageLayer = [CALayer layer];
  imageLayer.frame = CGRectMake(10.0f, 234.0f, 300.0f, 226.0f);

  imageLayer.transform = CATransform3DMakeRotation(-20.0f * M_PI / 180.0f,
                                                   1.0f, 0.0f, 0.0f);
  imageLayer.contents = (id)[[UIImage imageNamed:@"IMG_0089.png"] CGImage];
  [transformed addSublayer:imageLayer];
	
	// BJL: Add a perspective effect
	CATransform3D initialTransform = transformed.sublayerTransform;
	initialTransform.m34 = 1.0 / -500;
	transformed.sublayerTransform = initialTransform;
	// BJL: Enable multitouch
	self.multipleTouchEnabled = YES;
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	
	NSMutableSet *currentTouches = [[[event touchesForView:self] mutableCopy] autorelease];
    [currentTouches minusSet:touches];
	
	// New touches are not yet included in the current touches for the view
	NSSet *totalTouches = [touches setByAddingObjectsFromSet:[event touchesForView:self]];
	if ([totalTouches count] > 1)
	{
		startingTouchDistance = [self distanceBetweenTouches:totalTouches];
		previousScale = 1.0;
	}
	else
	{
		previousLocation = [[touches anyObject] locationInView:self];
	}
	
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	if ([[event touchesForView:self] count] > 1) // Pinch gesture
	{
		// BJL: The following is the pinch-scaling I used in Molecules
		// Scale using pinch gesture
		CGFloat newTouchDistance = [self distanceBetweenTouches:[event touchesForView:self]];
		CGFloat incrementalScaleFactor = (newTouchDistance / startingTouchDistance) / previousScale;
		CATransform3D currentTransform = transformed.sublayerTransform;
		
		CATransform3D scaleTransform = CATransform3DScale(currentTransform, incrementalScaleFactor, incrementalScaleFactor, incrementalScaleFactor);
		transformed.sublayerTransform = scaleTransform;
		previousScale = (newTouchDistance / startingTouchDistance);
		
	}
	else
	{
		CGPoint location = [[touches anyObject] locationInView:self];
		
		// BJL: The following is the code I used in Molecules to do 3-D rotation
		CATransform3D currentTransform = transformed.sublayerTransform;
		CGFloat displacementInX = location.x - previousLocation.x;
		CGFloat displacementInY = previousLocation.y - location.y;
		
		CGFloat totalRotation = sqrt(displacementInX * displacementInX + displacementInY * displacementInY);
		
		CATransform3D rotationalTransform = CATransform3DRotate(currentTransform, totalRotation * M_PI / 180.0, 
																((displacementInX/totalRotation) * currentTransform.m12 + (displacementInY/totalRotation) * currentTransform.m11), 
																((displacementInX/totalRotation) * currentTransform.m22 + (displacementInY/totalRotation) * currentTransform.m21), 
																((displacementInX/totalRotation) * currentTransform.m32 + (displacementInY/totalRotation) * currentTransform.m31));
		previousLocation = location;
		
		transformed.sublayerTransform = rotationalTransform;
	}
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	// BJL: If transitioning from a pinch gesture to a rotation, make sure the starting touch location is set properly
	NSMutableSet *remainingTouches = [[[event touchesForView:self] mutableCopy] autorelease];
    [remainingTouches minusSet:touches];
	if ([remainingTouches count] < 2)
	{
		previousLocation = [[remainingTouches anyObject] locationInView:self];
	}
	
}

- (void)dealloc {
  [super dealloc];
}


- (float)distanceBetweenTouches:(NSSet *)touches;
{
	int currentStage = 0;
	CGPoint point1, point2;
	
	
	for (UITouch *currentTouch in touches)
	{
		if (currentStage == 0)
		{
			point1 = [currentTouch locationInView:self];
			currentStage++;
		}
		else if (currentStage == 1) 
		{
			point2 = [currentTouch locationInView:self];
			currentStage++;
		}
		else
		{
		}
	}
	return (sqrt((point1.x - point2.x) * (point1.x - point2.x) + (point1.y - point2.y) * (point1.y - point2.y)));
}

@end
